
% phase space viewer
% Torsten Limberg / DESY, Hamburg
% Sascha Meykopff / DESY, Hamburg

function ps_viewer(varargin)
    data = struct();
    data = createGui( data );                       % open figure and create gui objects
    set( data.handles.psv_gui, 'UserData', data);   % store data structure in figure, now the handles are available in callbacks
    setupKeymap( data.handles );
    figureResize( data.handles.psv_gui );           % default size of object
    data = psv_initialize( data, data.handles );
    set( data.handles.psv_gui, 'CurrentAxes', data.handles.axes1 ); % current axes is axes no. 1
    data = psv_set_ui_visibility(data);             % hide elements
    set( data.handles.psv_gui, 'UserData', data);
    data =  setupCallbacks( data );
    renameMenuItems( data );
    set( data.handles.psv_gui, 'UserData', data);
    return;
end

function data = setupCallbacks( data )
    % panel 6
    set( data.handles.checkboxmesh_1,        'Callback', {@call_and_visibility, @psv_plot_ps, 1});
    set( data.handles.checkboxmesh_2,        'Callback', {@call_and_visibility, @psv_plot_ps, 2});
    set( data.handles.checkboxsquare_1,      'Callback', {@call_and_visibility, @psv_plot_ps, 1});
    set( data.handles.checkboxsquare_2,      'Callback', {@call_and_visibility, @psv_plot_ps, 2});
    set( data.handles.popupmenu_12,          'Callback', {@popupmenu_Callback, 1, 2, [ data.handles.popupmenu_11 data.handles.popupmenu_12 data.handles.togglebutton_slice_1 ] });
    set( data.handles.popupmenu_21,          'Callback', {@popupmenu_Callback, 2, 1, [ data.handles.popupmenu_21 data.handles.popupmenu_22 data.handles.togglebutton_slice_2 ] });
    set( data.handles.popupmenu_11,          'Callback', {@popupmenu_Callback, 1, 1, [ data.handles.popupmenu_11 data.handles.popupmenu_12 data.handles.togglebutton_slice_1 ] });
    set( data.handles.popupmenu_22,          'Callback', {@popupmenu_Callback, 2, 2, [ data.handles.popupmenu_21 data.handles.popupmenu_22 data.handles.togglebutton_slice_2 ] });
    set( data.handles.axes1 ,                'ButtonDownFcn', {@call_and_visibility, @setMyAxes, 1}); 
    set( data.handles.axes2 ,                'ButtonDownFcn', {@call_and_visibility, @setMyAxes, 2});

    % panel 7
    set( data.handles.edit_slicenr_1,        'Callback', {@edit_revert, data.handles.slider_slice_1} );
    set( data.handles.edit_slicenr_2,        'Callback', {@edit_revert, data.handles.slider_slice_2} );
    set( data.handles.edit_slice_1,          'Callback', {@edit_slice_Callback, 1, data.handles.slider_slice_1});
    set( data.handles.edit_slice_2,          'Callback', {@edit_slice_Callback, 2, data.handles.slider_slice_2});
    set( data.handles.slider_slice_1,        'Callback', {@slider_slice_Callback,data.handles.edit_slice_1,1});
    set( data.handles.slider_slice_2,        'Callback', {@slider_slice_Callback,data.handles.edit_slice_2,2});
    set( data.handles.togglebutton_slice_1,  'Callback', {@call_and_visibility, @psv_plot_ps, 1});
    set( data.handles.togglebutton_slice_2,  'Callback', {@call_and_visibility, @psv_plot_ps, 2});
    
    % panel movie
    set( data.handles.slider_files_1,        'Callback', {@slider_files_Callback,1} );
    set( data.handles.slider_files_2,        'Callback', {@slider_files_Callback,2} );
    set( data.handles.edit_dir_1,            'Callback', {@edit_revert, data.handles.slider_files_1});
    set( data.handles.edit_dir_2,            'Callback', {@edit_revert, data.handles.slider_files_2});
    set( data.handles.pushbutton_movie_1,    'Callback', {@call_and_visibility, @psv_show_movie, 1, 0});
    set( data.handles.pushbutton_movie_2,    'Callback', {@call_and_visibility, @psv_show_movie, 2, 0});
    set( data.handles.pushbutton_save_movie_1,'Callback', {@call_and_visibility, @psv_show_movie, 1, 1});
    set( data.handles.pushbutton_save_movie_2,'Callback', {@call_and_visibility, @psv_show_movie, 2, 1});
    
    % menus
    set( data.handles.File_menu_new,             'Callback', {@call_and_visibility, @psv_ps_generator } );
    set( data.handles.File_menu_open,            'Callback', {@call_and_visibility, @psv_get_ps, false, true });
    set( data.handles.File_menu_save_as,         'Callback', {@call_and_visibility, @psv_save } );
    set( data.handles.File_menu_quit,            'Callback', @File_menu_quit_Callback  );
    set( data.handles.Edit_menu_Twiss_Parameters,'Callback', {@call_and_visibility, @psv_edit_optics } );
    set( data.handles.Edit_menu_RF_Chirp,        'Callback', {@call_and_visibility, @psv_edit_chirp } );
    set( data.handles.Plot_Menue_Sum,            'Callback', {@call_and_visibility, @psv_sum_plot } );
    set( data.handles.Plot_Menue_Mesh,           'Callback', {@call_and_visibility, @psv_all_mesh_plots } );
    set( data.handles.poltminchi,                'Callback', {@call_and_visibility, @psv_plot_ming_chi } );
    set( data.handles.axes_in_seperate_window,   'Callback', {@call_and_visibility, @psv_print_axes } );
    set( data.handles.Options_Menue_Plot_Plot,   'Callback', {@call_and_visibility, @psv_set_options, 'Plot_Plot'} );
    set( data.handles.Options_Menue_Plot_Sum,    'Callback', {@call_and_visibility, @psv_set_options, 'Plot_Sum'} );
    set( data.handles.Options_Menue_Plot_Mesh,   'Callback', {@call_and_visibility, @psv_set_options, 'Plot_Mesh'} );
    set( data.handles.Options_Sub_Disp,          'Callback', @Options_Sub_Disp_Callback);
    set( data.handles.help_keymap,               'Callback', {@call_and_visibility, @help_keymap} );
    data.keyDownList = { ...
        'c' data.handles.File_menu_new              'control' [] 'Create new phase space'; ...
        'o' data.handles.File_menu_open             'control' [] 'Open file'; ...
        'q' data.handles.File_menu_quit             'control' [] 'Quit'; ...
        'r' data.handles.Edit_menu_RF_Chirp         'control' [] 'Edit rf chirp'; ...
        't' data.handles.Edit_menu_Twiss_Parameters 'control' [] 'Edi twiss parameters'; ...
        's' data.handles.Plot_Menue_Sum             'control' [] 'Plot summary'; ...
        'p' data.handles.Plot_Menue_Mesh            'control' [] 'Open "mesh" plot'; ...
        'e' data.handles.axes_in_seperate_window    'control' [] 'Open axes in  seperate figure'; ...
        'm' data.handles.poltminchi                 'control' [] 'Open SSY/Ming Chi plot'; ...
        'g' data.handles.Options_Menue_Plot_Plot    'control' [] 'General plot options'; ...
        'k' data.handles.Options_Menue_Plot_Sum     'control' [] 'Summary plot options'; ...
        'h' data.handles.Options_Menue_Plot_Mesh    'control' [] '"mesh" plot options'; ...
        'o' data.handles.axes1                      []        @aktivateOtherAxes 'Toggle active axes'; ...
        '1' data.handles.axes1                      'alt'     @aktivateAxes1 'Activate axes 1'; ...
        '2' data.handles.axes2                      'alt'     @aktivateAxes2 'Activate axes 2' ; ...
        'm' data.handles.checkboxmesh_1             []        @aktivateCurrentAxes 'Mesh plot (on/off)'; ...
        'f' data.handles.checkboxsquare_1           []        @aktivateCurrentAxes 'Fixed axis (on/off)'; ...
        's' data.handles.togglebutton_slice_1       []        @aktivateCurrentAxes 'Slice (on/off)'; ...
        'x' data.handles.popupmenu_11               []        @aktivateCurrentAxes 'Change X axis'; ...
        'y' data.handles.popupmenu_21               []        @aktivateCurrentAxes 'Change Y axis'; ...
        'a' data.handles.slider_slice_1             []        @currentAxesSliderLeft 'Slice on step left'; ...
        'd' data.handles.slider_slice_1             []        @currentAxesSliderRight 'Slice on step right'; ...
        'p' data.handles.pushbutton_movie_1         []        @aktivateCurrentAxes 'Play movie'; ...
        'w' data.handles.slider_files_1             []        @currentAxesSliderLeft 'Previous file'; ...
        'e' data.handles.slider_files_1             []        @currentAxesSliderRight 'Next file'; ...
        'h' data.handles.help_keymap                []        [] 'About keymap' ; ...
        };
    
end

%%% callback funtions:

% --------------------------------------------------------------------
function Options_Sub_Disp_Callback(hObject, eventdata, handles) %#ok<INUSD>
    data = get( get( hObject, 'UserData'), 'UserData');
    if(strcmp(get(hObject,'Checked'),'on'))
        set(hObject,'Checked','off');
        data.psv_options(1).plot.subtractDispersion = 0;
        data.psv_options(2).plot.subtractDispersion = 0;
    else
        set(hObject,'Checked','on');
        data.psv_options(1).plot.subtractDispersion = 1;
        data.psv_options(2).plot.subtractDispersion = 1;
    end
    set( get( hObject, 'UserData'), 'UserData', data);
end

% --------------------------------------------------------------------
function File_menu_quit_Callback(hObject, eventdata, handles) %#ok<INUSD>
    if isdeployed
        close all
        quit force
    else
        close all
    end
end

function data = setMyAxes(data, newAxes)
    if newAxes ~= 0
        data.my_axes = newAxes;
    end
end

% --- Executes on slider movement.
function slider_files_Callback(hObject, eventdata, myaxes) %#ok<INUSL>
    data = get( get( hObject, 'UserData'), 'UserData');
    data.my_axes = myaxes;
    new_pos = round(get(hObject,'Value'));
    data.ps(data.my_axes).bl.act_file = new_pos;
    data = psv_get_ps( data, true, false);
    data = psv_set_ui_visibility( data );
    set( get( hObject, 'UserData'), 'UserData', data);
end

% --- Executes during object creation, after setting all properties.
function slider_CreateFcn(hObject, eventdata, bgCol) %#ok<INUSL>
    if isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
        set(hObject,'BackgroundColor', bgCol);
    end
end

% --- Executes on selection change in popupmenu
function popupmenu_Callback(hObject, eventdata, myaxes, hv, guiHandles) %#ok<INUSL>
    data = get( get( hObject, 'UserData'), 'UserData');    
    objectValue = get(hObject,'Value');
    if hv == 2
        data.ps(myaxes).hor_axis = objectValue;
    else
        data.ps(myaxes).ver_axis = objectValue;
    end
    
    % check if hor_axis/ver_axis combination is allowed. update if not
    % we don't need to redraw later (after psv_set_ui_vis) if we do this now
    hor_axis = data.ps(myaxes).hor_axis;
    ver_axis = data.ps(myaxes).ver_axis;
    
    isSlice  = false;
    
    if data.psv_hnames.isSliceable(data, myaxes) && get(guiHandles(3),'Value')
        isSlice   = true;
    end
    [popupString1 popupString2] = data.psv_hnames.getPopupEntries(data, myaxes, isSlice);
    if (ver_axis > length(popupString1))
        % update value if out of range
        data.ps(myaxes).ver_axis = length(popupString1);
        set(guiHandles(1),'Value',length(popupString1));
    end
    if (hor_axis > length(popupString2))
        % update value if out of range
        data.ps(myaxes).hor_axis = length(popupString2);
        set(guiHandles(2),'Value',length(popupString2));
    end
    data = psv_plot_ps(data, myaxes);
    data = psv_set_ui_visibility( data );
    set( get( hObject, 'UserData'), 'UserData', data);
end

% --- Executes on slider movement.
function slider_slice_Callback(hObject, eventdata, edit_handle, myaxes) %#ok<INUSL>
    data = get( get( hObject, 'UserData'), 'UserData');    
    new_pos = round(get(hObject,'Value'));
    set(edit_handle,'String',num2str(new_pos))
    set(hObject, 'Value', new_pos);
    data = psv_set_ui_visibility( data, 'updateSliceOnly');
    data = psv_plot_ps(data, myaxes);
    set( get( hObject, 'UserData'), 'UserData', data);
end

% general callback function with data structure support
function call_and_visibility( hObject, eventdata, callfunction, parameter1, parameter2 ) %#ok<INUSL>
    data = get( get( hObject, 'UserData'), 'UserData');    
    if nargin == 3
        data = callfunction(data);
    elseif nargin == 4 
        data = callfunction( data, parameter1 );
    elseif nargin == 5
        data = callfunction( data, parameter1, parameter2 );
    end
    data = psv_set_ui_visibility( data );
    set( get( hObject, 'UserData'), 'UserData', data);
end

% revert any changes in edit field
function edit_revert(hObject, eventdata, slider_handle) %#ok<INUSL>
    set(hObject, 'String', num2str( get( slider_handle, 'Max') ));
end

function edit_slice_Callback(hObject, eventdata, myaxes, slider_handle) %#ok<INUSL>
    data = get( get( hObject, 'UserData'), 'UserData');    
    newValue = get(hObject, 'String');
    min = get( slider_handle, 'Min');
    max = get( slider_handle, 'Max');
    newValue = str2double(newValue);
    
    if isnan(newValue)||(newValue < min)||(newValue > max)
        newValue = get( slider_handle, 'Value');
        set( hObject, 'String', num2str(newValue) );
        return;
    end
    set( slider_handle, 'Value', newValue);
    data = psv_set_ui_visibility( data, 'updateSliceOnly');
    data = psv_plot_ps(data, myaxes);
    set( get( hObject, 'UserData'), 'UserData', data);
end

function data = help_keymap( data )
    msg = cell(size( data.keyDownList,1 ),1);
    for i=1 : size( data.keyDownList,1 )
        if ~isempty(data.keyDownList{i,3})
            msg{i,1} = [ '(' upper(char(data.keyDownList{i,1})) '+' upper(char(data.keyDownList{i,3})) ') ' char(data.keyDownList{i,5}) ];
        else
            msg{i,1} = [ upper(char(data.keyDownList{i,1})) ': ' char(data.keyDownList{i,5}) ];
        end
    end
    msgbox(msg,'PS Viewer keymapping');
end

function hObject = aktivateOtherAxes( data, hObject ) %#ok<INUSD>
    ca = get( data.handles.psv_gui, 'CurrentAxes' );
    if ca == data.handles.axes1
        hObject = data.handles.axes2;
    else
        hObject = data.handles.axes1;
    end
    set( data.handles.psv_gui, 'CurrentAxes', hObject );
end

function hObject = aktivateAxes1( data, hObject ) %#ok<INUSD>
    hObject = data.handles.axes1;
    set( data.handles.psv_gui, 'CurrentAxes', hObject );
end

function hObject = aktivateAxes2( data, hObject ) %#ok<INUSD>
    hObject = data.handles.axes2;
    set( data.handles.psv_gui, 'CurrentAxes', hObject );
end

function hObject = aktivateCurrentAxes( data, hObject ) %#ok<INUSL>
    currentTag = get( hObject, 'Tag' );
    axesTag = get( gca, 'Tag' );
    axesNum = axesTag( length(axesTag):length(axesTag) );
    workingTag = [ currentTag( 1:length(currentTag)-1) axesNum ];
    hObject = findobj( 'Tag', workingTag );
    
    % this is an ugly bugfix because the popupmenu tags are named disadvantageous
    if strcmp( get( hObject, 'Type'), 'uicontrol' ) && strcmp( get( hObject, 'Style'), 'popupmenu' )
        l = length(currentTag);
        hObject = findobj( 'Tag', [ currentTag( 1:l-2 ) axesNum currentTag( l-1:l-1 ) ] );
    end
    % bugfix end
    
    if strcmp( get( hObject, 'Visible'), 'off')
        return;
    end
    if strcmp( get( hObject, 'Type'), 'uicontrol' )
        if strcmp( get( hObject, 'Style'), 'checkbox' ) || strcmp( get( hObject, 'Style'), 'togglebutton' )
            value = get( hObject, 'Value');
            if value==0
                value = 1;
            else
                value = 0;
            end
            set( hObject, 'Value', value);
        elseif strcmp( get( hObject, 'Style'), 'popupmenu' )
            max = size( get( hObject, 'String' ), 1);
            value = get( hObject, 'Value');
            if value+1 > max
                value = 0;
            end
            set( hObject, 'Value', value+1);
        end
    end
end

function hObject = currentAxesSliderLeft( data, hObject ) %#ok<INUSL>
    currentTag = get( hObject, 'Tag' );
    axesTag = get( gca, 'Tag' );
    axesNum = axesTag( length(axesTag):length(axesTag) );
    workingTag = [ currentTag( 1:length(currentTag)-1) axesNum ];
    hObject = findobj( 'Tag', workingTag );
    value = get( hObject, 'Value');
    min = get( hObject, 'Min');
    if value-1 < min 
        value = get( hObject, 'Max') + 1;
    end
    set( hObject, 'Value', value - 1);
end

function hObject = currentAxesSliderRight( data, hObject ) %#ok<INUSL>
    currentTag = get( hObject, 'Tag' );
    axesTag = get( gca, 'Tag' );
    axesNum = axesTag( length(axesTag):length(axesTag) );
    workingTag = [ currentTag( 1:length(currentTag)-1) axesNum ];
    hObject = findobj( 'Tag', workingTag );
    value = get( hObject, 'Value');
    max = get( hObject, 'Max');
    if value+1 > max
        value = get( hObject, 'Min') - 1;
    end
    set( hObject, 'Value', value + 1);
end

function buttonDown( hObject, eventdata )
    type = get( hObject, 'Type' );
    if strcmp(eventdata.Key, 'control') ; return; end;
    while ( ~strcmp(type, 'figure') ) % get root figure
        hObject = get( hObject, 'UserData');
        type = get( hObject, 'Type' );        
    end
    data = get( hObject, 'UserData' ); % get data structure
    for i=1 : size( data.keyDownList, 1 )
        if strcmp( eventdata.Key, char(data.keyDownList{i,1}))
            if ( isempty( data.keyDownList{i,3} ) && isempty(eventdata.Modifier) ) || ...
                ( strcmp( data.keyDownList{i,3}, char(eventdata.Modifier) ) )
%                display( [ 'buttonDown: ' type ' ' tag ' ' eventdata.Key ' ' eventdata.Modifier ]);
                hObject = data.keyDownList{i,2};
                if ~isempty( data.keyDownList{i,4} ) 
                    hObject = data.keyDownList{i,4}( data,hObject );
                end
                type = get( hObject, 'Type' );
                if strcmp( type, 'axes' )
                    btdn = get( hObject, 'ButtonDownFcn' );
                else
                    btdn = get( hObject, 'Callback' );
                end
                if  size( btdn,1 ) == 1
                    btdn( hObject );
                elseif  size( btdn,1 ) == 2
                    btdn{1}( hObject, [], btdn{2} );
                elseif  size( btdn,1 ) == 3
                    btdn{1}( hObject, [], btdn{2}, btdn{3} );
                elseif  size( btdn,1 ) == 4
                    btdn{1}( hObject, [], btdn{2}, btdn{3}, btdn{4} );
                elseif  size( btdn,1 ) == 5
                    btdn{1}( hObject, [], btdn{2}, btdn{3}, btdn{4}, btdn{5} );
                end
                return;
            end;
        end; 
    end
end

%%% gui init / resize functions:

function [ data ] = createGui( data )
    handles = struct();
    colordef white;
    fig = figure('Color', get(0,'defaultUicontrolBackgroundColor'), 'MenuBar', 'none', 'Units', 'Pixels', 'Resize', 'On', 'ResizeFcn', @figureResize, 'NumberTitle', 'off');

    handles.uipanel2 = uipanel('Parent',fig, 'Title','Beam Parameters:', 'Position',[.25 .1 .5 .8]);
    handles.uipanel6 = uipanel('Parent',fig, 'Title','Beam Display', 'Position',[.25 .1 .5 .8]);
    handles.uipanel7 = uipanel('Parent',fig, 'Title','View Phase Space in Longitudinal Slices ', 'Position',[.25 .1 .5 .8]);
    handles.uipanel5 = uipanel('Parent',fig, 'Title','Phase Space Data Files Along the Beam Line', 'Position',[.25 .1 .5 .8]);
    handles.text_message = uicontrol( fig, 'Style','edit', 'String', {'' '' '' '' '' '' '' '' '' '' }, 'Position',[30 50 130 20], 'CreateFcn', { @slider_CreateFcn, 'white'}, 'ListboxTop', 1.0, 'Max',10.0, 'Min',0.0);

    % panel 2
    handles.text_emit_1  = uicontrol( handles.uipanel2, 'Style','text', 'String','Emittance:', 'Position',[30 50 130 20], 'HorizontalAlignment', 'left');
    handles.text_twiss_1 = uicontrol( handles.uipanel2, 'Style','text', 'String','Twiss:', 'Position',[30 50 130 20], 'HorizontalAlignment', 'left');
    handles.text_emit_2  = uicontrol( handles.uipanel2, 'Style','text', 'String','Emittance:', 'Position',[30 50 130 20], 'HorizontalAlignment', 'left');
    handles.text_twiss_2 = uicontrol( handles.uipanel2, 'Style','text', 'String','Twiss:', 'Position',[30 50 130 20], 'HorizontalAlignment', 'left');

    % panel 6
    handles.text_RMS_12  = uicontrol( handles.uipanel6, 'Style','text', 'String','RMS:', 'Position',[30 50 130 20], 'HorizontalAlignment', 'left');
    handles.text_RMS_11  = uicontrol( handles.uipanel6, 'Style','text', 'String','RMS:', 'Position',[30 50 130 20], 'HorizontalAlignment', 'left');
    handles.text_RMS_21  = uicontrol( handles.uipanel6, 'Style','text', 'String','RMS:', 'Position',[30 50 130 20], 'HorizontalAlignment', 'left');
    handles.text_RMS_22  = uicontrol( handles.uipanel6, 'Style','text', 'String','RMS:', 'Position',[30 50 130 20], 'HorizontalAlignment', 'left');
    handles.checkboxmesh_1 = uicontrol(handles.uipanel6, 'Style','checkbox', 'String','Mesh', 'Value',0,'Position',[30 20 130 20] );
    handles.checkboxmesh_2 = uicontrol(handles.uipanel6, 'Style','checkbox', 'String','Mesh', 'Value',0,'Position',[30 20 130 20] );
    handles.checkboxsquare_1  = uicontrol(handles.uipanel6, 'Style','checkbox', 'String','Fix', 'Value',0,'Position',[30 20 130 20] );
    handles.checkboxsquare_2  = uicontrol(handles.uipanel6, 'Style','checkbox', 'String','Fix', 'Value',0,'Position',[30 20 130 20] );
    handles.popupmenu_12 = uicontrol(handles.uipanel6, 'Style','popupmenu', 'String',{'','','','','',''},'Value',1,'Position',[30 80 130 20] );
    handles.popupmenu_21 = uicontrol(handles.uipanel6, 'Style','popupmenu', 'String',{'','','','','',''},'Value',1,'Position',[30 80 130 20] );
    handles.popupmenu_11 = uicontrol(handles.uipanel6, 'Style','popupmenu', 'String',{'','','','','',''},'Value',1,'Position',[30 80 130 20] );
    handles.popupmenu_22 = uicontrol(handles.uipanel6, 'Style','popupmenu', 'String',{'','','','','',''},'Value',1,'Position',[30 80 130 20] );
    handles.axes1 = axes('Parent',handles.uipanel6, 'Position',[.15 .15 .7 .7] );
    handles.axes2 = axes('Parent',handles.uipanel6, 'Position',[.15 .15 .7 .7] );
    
    % panel 7
    handles.edit_slicenr_1 = uicontrol(handles.uipanel7, 'Style','edit', 'String','', 'Position',[30 50 130 20], 'BackgroundColor', 'white' );
    handles.edit_slicenr_2 = uicontrol(handles.uipanel7, 'Style','edit', 'String','', 'Position',[30 50 130 20], 'BackgroundColor', 'white' );
    handles.edit_slice_1 = uicontrol(handles.uipanel7, 'Style','edit', 'String','', 'Position',[30 50 130 20], 'BackgroundColor', 'white' );
    handles.edit_slice_2 = uicontrol(handles.uipanel7, 'Style','edit', 'String','', 'Position',[30 50 130 20], 'BackgroundColor', 'white' );
    handles.text_slicenr_1  = uicontrol( handles.uipanel7, 'Style','text', 'String','# of Slices', 'Position',[30 50 130 20], 'HorizontalAlignment', 'left', 'FontSize', 7);
    handles.text_slicenr_2  = uicontrol( handles.uipanel7, 'Style','text', 'String','# of Slices', 'Position',[30 50 130 20], 'HorizontalAlignment', 'left', 'FontSize', 7);
    handles.text_slice_1  = uicontrol( handles.uipanel7, 'Style','text', 'String','Slice #', 'Position',[30 50 130 20], 'HorizontalAlignment', 'left', 'FontSize', 7);
    handles.text_slice_2  = uicontrol( handles.uipanel7, 'Style','text', 'String','Slice #', 'Position',[30 50 130 20], 'HorizontalAlignment', 'left', 'FontSize', 7);
    handles.slider_slice_1 = uicontrol(handles.uipanel7, 'Style','slider', 'Max',2,'Min',1,'Value',1, 'SliderStep',[0.05 0.2], 'Position',[30 20 150 30], 'CreateFcn', { @slider_CreateFcn, [.9 .9 .9]} );
    handles.slider_slice_2 = uicontrol(handles.uipanel7, 'Style','slider', 'Max',2,'Min',1,'Value',1, 'SliderStep',[0.05 0.2], 'Position',[30 20 150 30], 'CreateFcn', { @slider_CreateFcn, [.9 .9 .9]} );
    handles.togglebutton_slice_1 = uicontrol(handles.uipanel7, 'Style','togglebutton', 'String','Slice', 'Value',0,'Position',[30 20 100 30] );
    handles.togglebutton_slice_2 = uicontrol(handles.uipanel7, 'Style','togglebutton', 'String','Slice', 'Value',0,'Position',[30 20 100 30] );

    % panel 5 (movie)
    handles.slider_files_1 = uicontrol(handles.uipanel5, 'Style','slider', 'Max',2,'Min',1,'Value',1, 'SliderStep',[0.05 0.2], 'Position',[30 20 150 30], 'CreateFcn', { @slider_CreateFcn, [.9 .9 .9]} );
    handles.slider_files_2 = uicontrol(handles.uipanel5, 'Style','slider', 'Max',2,'Min',1,'Value',1, 'SliderStep',[0.05 0.2], 'Position',[30 20 150 30], 'CreateFcn', { @slider_CreateFcn, [.9 .9 .9]} );
    handles.edit_dir_1 = uicontrol(handles.uipanel5, 'Style','edit', 'String','', 'Position',[30 50 130 20], 'BackgroundColor', 'white' );
    handles.edit_dir_2 = uicontrol(handles.uipanel5, 'Style','edit', 'String','', 'Position',[30 50 130 20], 'BackgroundColor', 'white' );
    handles.pushbutton_movie_1 = uicontrol(handles.uipanel5, 'Style','pushbutton', 'String','Movie', 'Value',0,'Position',[30 20 50 30] );
    handles.pushbutton_movie_2 = uicontrol(handles.uipanel5, 'Style','pushbutton', 'String','Movie', 'Value',0,'Position',[30 20 100 30] );
    handles.text_file_1  = uicontrol( handles.uipanel5, 'Style','text', 'String','File #', 'Position',[30 50 130 20], 'HorizontalAlignment', 'left', 'FontSize', 7);
    handles.text_file_2  = uicontrol( handles.uipanel5, 'Style','text', 'String','File #', 'Position',[30 50 130 20], 'HorizontalAlignment', 'left', 'FontSize', 7);
    handles.pushbutton_save_movie_1 = uicontrol(handles.uipanel5, 'Style','pushbutton', 'String','.', 'Value',0,'Position',[120 20 10 30] );
    handles.pushbutton_save_movie_2 = uicontrol(handles.uipanel5, 'Style','pushbutton', 'String','.', 'Value',0,'Position',[120 20 10 30] );
    
    % menus
    handles.psv_File                    = uimenu( fig,                          'Label','File' );
    handles.File_menu_new               = uimenu( handles.psv_File,             'Label','New...' );
    handles.File_menu_open              = uimenu( handles.psv_File,             'Label','Open...' );
    handles.File_menu_save_as           = uimenu( handles.psv_File,             'Label','Save As...' );
    handles.File_menu_quit              = uimenu( handles.psv_File,             'Label','Quit' );
    handles.psv_Edit                    = uimenu( fig,                          'Label','Edit' );
    handles.Edit_menu_Twiss_Parameters  = uimenu( handles.psv_Edit,             'Label','Twiss Parameters' );
    handles.Edit_menu_RF_Chirp          = uimenu( handles.psv_Edit,             'Label','RF Chirp' );
    handles.Plot_Menue                  = uimenu( fig,                          'Label','Plots' );
    handles.Plot_Menue_Sum              = uimenu( handles.Plot_Menue,           'Label','Summary Plot' );
    handles.Plot_Menue_Mesh             = uimenu( handles.Plot_Menue,           'Label','Mesh Plot' );
    handles.axes_in_seperate_window     = uimenu( handles.Plot_Menue,           'Label','Put axes in seperate window' );
    handles.poltminchi                  = uimenu( handles.Plot_Menue,           'Label','SSY/Ming Chi plot' );
    handles.Options                     = uimenu( fig,                          'Label','Options' );
    handles.Options_Menue_Plot          = uimenu( handles.Options,              'Label','Plotting Options' );
    handles.Options_Menue_Plot_Plot     = uimenu( handles.Options_Menue_Plot,   'Label','General Plot Options' );
    handles.Options_Menue_Plot_Sum      = uimenu( handles.Options_Menue_Plot,   'Label','SumPlot Options' );
    handles.Options_Menue_Plot_Mesh     = uimenu( handles.Options_Menue_Plot,   'Label','SSY/Ming Chi Plot Options' );
    handles.Options_Sub_Disp            = uimenu( handles.Options,              'Label','Subtract Dispersion' );
    handles.help_keymap                 = uimenu( handles.Options,              'Label','Help keymap', 'Separator', 'on' );

    names = fieldnames( handles );
    for i = 1 : length( names )
        if ( strcmp( get(handles.(names{i}), 'Type') , 'uimenu') )
            set( handles.(names{i}), 'Tag', names{i}, 'UserData', fig );
        else
            set( handles.(names{i}), 'Units', 'Pixels', 'Tag', names{i}, 'UserData', fig );
        end
    end
    handles.psv_gui = fig;
    data.handles = handles;
    
    fpos = get( fig, 'Position');
    fpos(4) = ((fpos(3) - 20 ) / 2) + 350;
    screen = get( 0, 'ScreenSize' );
    if fpos(2) + fpos(4) > screen(4)
        fpos(2) = screen(4) - fpos(4) - 100;
    end
    set( fig, 'Position', fpos );
    
end

function setupKeymap( handles )
    handlenames = fieldnames( handles );
    for i = 1 : size( handlenames )
        name = char( handlenames(i) );
        type = get( handles.(name), 'Type' );
        if strcmp( type, 'figure') || strcmp( type, 'uicontrol') 
            set(handles.(name), 'KeyPressFcn', @buttonDown);
        end
    end
end

function renameMenuItems( data )
    for i=1 : size(data.keyDownList,1)
        if strcmp( get( data.keyDownList{i,2}, 'Type'), 'uimenu' )
            label = get( data.keyDownList{i,2}, 'Label') ;
            if strcmp(char(data.keyDownList{i,3}), 'control')
                label = [ label ' (X+' upper(char(data.keyDownList{i,1})) ')' ]; %#ok<AGROW>
            elseif isempty(data.keyDownList{i,3})
                label = [ label ' (' upper(char(data.keyDownList{i,1})) ')' ]; %#ok<AGROW>
            else
                label = [ label ' (' char(data.keyDownList{i,3}) '+' upper(char(data.keyDownList{i,1})) ')' ]; %#ok<AGROW>
            end
            set( data.keyDownList{i,2}, 'Label', label) ;
        end
    end
end

function figureResize(hObject, eventdata, handles) %#ok<INUSD>
    try
        data = get( hObject, 'UserData' );
        if isempty(data); return; end;
        h = data.handles; % shortcut
        lOff = 10; rOff = 10; bOff = 10; hSpace = 10; tOff = 10; vSpace = 10;
        fpos = get( hObject, 'Position');
        hStart = (fpos(3) / 2) - (hSpace * 2);
        set( h.text_message, 'Position', [ lOff bOff fpos(3)-(lOff+rOff) 60 ] );
        bOff = bOff + hSpace + 60;
        set( h.uipanel2, 'Position', [ lOff bOff fpos(3)-(lOff+rOff) 60 ] );
        set( h.text_twiss_1, 'Position', [ lOff    2 (fpos(3)-(lOff+rOff+50))/2 20 ] );
        set( h.text_emit_1, 'Position',  [ lOff   22 (fpos(3)-(lOff+rOff+50))/2 20 ] );
        set( h.text_twiss_2, 'Position', [ hStart  2 (fpos(3)-(lOff+rOff+50))/2 20 ] );
        set( h.text_emit_2, 'Position',  [ hStart 22 (fpos(3)-(lOff+rOff+50))/2 20 ] );
                
        bOff = bOff + hSpace + 60;
        set( h.uipanel5, 'Position', [ lOff fpos(4)-(60+tOff) fpos(3)-(lOff+rOff) 60 ] );
        set( h.slider_files_1, 'Position', [ lOff 10 (hStart-100-3*lOff) 20 ] );
        set( h.edit_dir_1, 'Position',  [ hStart-100-2*lOff 10 50 20 ] );
        set( h.text_file_1, 'Position', [ hStart-100-2*lOff 30 50 10 ] );
        set( h.pushbutton_movie_1, 'Position', [ hStart-50-2*lOff 10 50 20 ] );
        set( h.pushbutton_save_movie_1, 'Position', [ hStart-50-2*lOff+50 10 10 20 ] );

        set( h.slider_files_2, 'Position', [ hStart+lOff 10 (hStart-100-lOff) 20 ] );
        set( h.edit_dir_2, 'Position',  [ hStart*2-100 10 50 20 ] );
        set( h.text_file_2, 'Position', [ hStart*2-100 30 50 10 ] );
        set( h.pushbutton_movie_2, 'Position', [ hStart*2-50 10 50 20 ] );
        set( h.pushbutton_save_movie_2, 'Position', [ hStart*2-50+50 10 10 20 ] );
        
        tOff = tOff + hSpace + 60;
        set( h.uipanel7, 'Position', [ lOff fpos(4)-(60+tOff) fpos(3)-(lOff+rOff) 60 ] );
        set( h.edit_slicenr_1, 'Position', [ lOff 10 50 20 ] );
        set( h.text_slicenr_1, 'Position', [ lOff 30 50 10 ] );
        set( h.slider_slice_1, 'Position', [ lOff+50 10 (hStart-150-3*lOff) 20 ] );
        set( h.edit_slice_1, 'Position',   [ hStart-100-2*lOff 10 50 20 ] );
        set( h.text_slice_1, 'Position',   [ hStart-100-2*lOff 30 50 10 ] );
        set( h.togglebutton_slice_1, 'Position', [ hStart-50-2*lOff 10 60 20 ] );
        
        set( h.edit_slicenr_2, 'Position', [ hStart+lOff 10 50 20 ] );
        set( h.text_slicenr_2, 'Position', [ hStart+lOff 30 50 10 ] );
        set( h.slider_slice_2, 'Position', [ hStart+50+lOff 10 (hStart-150-lOff) 20 ] );
        set( h.edit_slice_2, 'Position',   [ hStart*2-100 10 50 20 ] );
        set( h.text_slice_2, 'Position',   [ hStart*2-100 30 50 10 ] );
        set( h.togglebutton_slice_2, 'Position', [ hStart*2-50 10 60 20 ] );

        tOff = tOff + hSpace + 60;
        set( h.uipanel6, 'Position', [ lOff bOff fpos(3)-(lOff+rOff) fpos(4)-(bOff+tOff) ] );
        set( h.checkboxmesh_1, 'Position', [ lOff lOff 80 20 ] );
        set( h.checkboxsquare_1, 'Position', [ lOff+(80+vSpace) lOff 80 20 ] );
        set( h.text_RMS_12, 'Position', [ lOff lOff+20 (hStart-(2*vSpace))/2 20 ] );
        set( h.popupmenu_12, 'Position', [ lOff+2*(80+vSpace) lOff 80 20 ] );
        
        set( h.checkboxmesh_2, 'Position', [ hStart lOff 80 20 ] );
        set( h.checkboxsquare_2, 'Position', [ hStart+(80+vSpace) lOff 80 20 ] );
        set( h.text_RMS_22, 'Position', [ hStart lOff+20 (hStart-(2*vSpace))/2 20 ] );
        set( h.popupmenu_22, 'Position', [ hStart+2*(80+vSpace) lOff 80 20 ] );

        set( h.popupmenu_11, 'Position', [ lOff fpos(4)-(bOff+tOff+2*hSpace+20) 80 20 ] );
        set( h.text_RMS_11, 'Position', [ hStart-(lOff+((hStart-(2*vSpace))/2)) fpos(4)-(bOff+tOff+2*hSpace+20) (hStart-(2*vSpace))/2 20 ] );
        set( h.popupmenu_21, 'Position', [ hStart+lOff fpos(4)-(bOff+tOff+2*hSpace+20) 80 20 ] );
        set( h.text_RMS_21, 'Position', [ hStart*2-((hStart-(2*vSpace))/2) fpos(4)-(bOff+tOff+2*hSpace+20) (hStart-(2*vSpace))/2 20 ] );

        set( h.axes1, 'OuterPosition', [ lOff lOff+40 (hStart-(2*vSpace)) fpos(4)-(bOff+tOff+2*hSpace+40+20+20) ] );
        set( h.axes2, 'OuterPosition', [ hStart lOff+40 (hStart-(2*vSpace)) fpos(4)-(bOff+tOff+2*hSpace+40+20+20) ] );

        if isfield(data,'ps')
            if isfield(data.ps(1),'cbar_axes') && ~isempty(data.ps(1).cbar_axes)
               fpos = get( h.axes1, 'Position');
               set(data.ps(1).cbar_axes, 'Position', [ fpos(1)+fpos(3)+2 fpos(2)+10 12 fpos(4)-20 ])
            end
            if isfield(data.ps(2),'cbar_axes') && ~isempty(data.ps(2).cbar_axes)
               fpos = get( h.axes2, 'Position');
               set(data.ps(2).cbar_axes, 'Position', [ fpos(1)+fpos(3)+2 fpos(2)+10 12 fpos(4)-20 ])
            end
        end
        
    catch
        display('exception in figureResize()');
    end
end
