function data = psv_all_mesh_plots( data ) 
%
% calculate sigma_s, sigma_delta, beta_x, emit_x, beta_y, emit_y along the
% mesh
%
if nargin ~= 1; return; end;

% if field ps.bl.file_names is shorter than 3, no mesh plot done 
if(isstruct(data.ps(data.my_axes)))
    if(~isfield(data.ps(data.my_axes),'bl'))
        uiwait(msgbox('No files for mesh plots found!','PSV_ALL_MESH_PLOTS','error'))
        return
    else
        if isempty(data.ps(data.my_axes).bl)
            psv_message(data, 'there are no other files. function canceled',0);
            return;
        end
        if(length(data.ps(data.my_axes).bl.file_names) < 3)
            uiwait(msgbox('No mesh plot for less than 3 files!','PSV_ALL_MESH_PLOTS','error'))
            return
        end
    end
else
    uiwait(msgbox('No phase space data!','PSV_ALL_MESH_PLOTS','error'))
    return
end
%
ps_files = data.ps(data.my_axes).bl.file_names;
mon_pos  = data.ps(data.my_axes).bl.mon_pos;
%
if  isempty(data.ps(data.my_axes).mesh_data)

    for file_nr=1:length(ps_files)
        disp(['# of file processed: ' num2str(file_nr)])
        filename = char(ps_files(file_nr));
        % put filename in structure
        data.ps(data.my_axes).filename = filename;
        data = psv_get_ps( data, false, false);

        gammag      = data.ps(data.my_axes).gamma;
        b_rms       = data.ps(data.my_axes).b_stat.b_rms;
        b_emit      = data.ps(data.my_axes).b_stat.b_emit;
        b_twiss     = data.ps(data.my_axes).b_stat.b_twiss;
        grid        = data.ps(data.my_axes).b_stat.grid;

        % find maximum and half-maximum positions
        Bshape = data.ps(data.my_axes).Bcurrent(5).shape;
        [peak_current i_max] = max(Bshape(:,2));
        b_center = Bshape(i_max,1);
        % find grid index of center
        [dummy center_grid_index] = min(abs(b_center-grid));
        if(b_center - grid(center_grid_index) <= 0);
            center_grid_index = center_grid_index -1;
        end
        
        % look for slice emittance
        csl_emit = data.ps(data.my_axes).slices(center_grid_index).b_stat.b_emit(1);
        % store in mesh_data
        mesh_data(file_nr,:)   =  [b_rms, b_emit(1), csl_emit ...
            ,b_emit(5),b_twiss(2,:)];         %#ok<AGROW>
        data.ps(data.my_axes).mesh_data = mesh_data;
        current(file_nr)=peak_current; %#ok<AGROW>
        data.ps(data.my_axes).bl.current = current;
        i_max(file_nr)=i_max; %#ok<NASGU>
    end
    
    data = psv_plot_ps(data, data.my_axes);

    data = psv_set_ui_visibility( data );
else
    mesh_data = data.ps(data.my_axes).mesh_data;
    current   = data.ps(data.my_axes).bl.current;
end
%
% make plot
figure('units','normalized','position',[.05 .05 .85 .85]);
%
subplot(3,2,1)
plot(mon_pos,mesh_data(:,5))
ylabel('Bunch Length [m]')
xlabel('Beamline Position [m]')

subplot(3,2,2)
plot(mon_pos,mesh_data(:,6))
ylabel('Energy Spread [rel.]')
xlabel('Beamline Position [m]')

subplot(3,2,3)
plot(mon_pos,mesh_data(:,10))
ylabel('Horizontal Beta Fkt. [m]')
xlabel('Beamline Position [m]')

subplot(3,2,4)
plot(mon_pos,mesh_data(:,7))
ylabel('Horizontal Emittance [m]')
xlabel('Beamline Position [m]')

subplot(3,2,5)
plot(mon_pos,mesh_data(:,8))
ylabel('Hor. Slice Emitt. (center) [m]')
xlabel('Beamline Position [m]')

subplot(3,2,6)

plot(mon_pos,current)
ylabel('Peak current [A]')
xlabel('Beamline Position [m]')

end