function [ cbar_axes ] = psv_meshplot_phasespace(ps, npoints, fix)
    if (fix == 1)
        edgeXmin = min(min(ps));
        edgeYmin = edgeXmin;
        edgeXmax = max(max(ps));
        edgeYmax = edgeXmax;
    else
        edgeXmin = min(ps(:,1));
        edgeYmin = min(ps(:,2));
        edgeXmax = max(ps(:,1));
        edgeYmax = max(ps(:,2));
    end
    offsetX = abs( edgeXmax - edgeXmin) * 0.05;
    offsetY = abs( edgeYmax - edgeYmin) * 0.05;
    edgeXmin = edgeXmin - offsetX;
    edgeXmax = edgeXmax + offsetX;
    edgeYmin = edgeYmin - offsetY;
    edgeYmax = edgeYmax + offsetY;
    edgesX   = edgeXmin: (edgeXmax-edgeXmin)/npoints :edgeXmax;
    edgesY   = edgeYmin: (edgeYmax-edgeYmin)/npoints :edgeYmax;
    I       = psmeshcount(ps, edgesX, edgesY);
    [X,Y]  = meshgrid(edgesX,edgesY);
    btnDown = get(gca,'ButtonDownFcn');
    userData = get(gca,'UserData');
    cTag = get( gca, 'Tag' );
    cUnits = get( gca, 'Units' );
    contourf(gca, X, Y, I', 'LineStyle', 'none', 'ButtonDownFcn', btnDown, 'UserData', userData);
    set( gca, 'UserData', userData);
    set( gca, 'ButtonDownFcn', btnDown);
    set( gca, 'Tag', cTag ); 
    set( gca, 'Units', cUnits );
    o = get(gca, 'Position');
    cbar_axes = colorbar('East');
    set( cbar_axes, 'Units', cUnits );
    set( cbar_axes, 'Tag', [cTag 'cbar'] );
    set( cbar_axes, 'Position', [ o(1)+o(3)+2 o(2)+10 12 o(4)-20 ])
    return;
end

function counterField = psmeshcount(ps, edgesX, edgesY)
    counterField = zeros(length(edgesX), length(edgesY));
    edgeXmin = edgesX(1) - 1;
    for psXi = 2:length(edgesX)
        % cut matrix into slices
        ps2    = ps( (ps(:,1) > edgeXmin)&(ps(:,1) <= edgesX(psXi)) , 2 );
        edgeYmin =  edgesY(1) -1;
        for psYi = 2:length(edgesY)
            % cut array into slices and count slice entries
            counterField(psXi-1, psYi-1) = length(ps2((ps2 > edgeYmin ) & (ps2 <= edgesY(psYi))));
            edgeYmin = edgesY(psYi);
        end
        edgeXmin = edgesX(psXi);
    end
end
