function [ data ] = psv_plot_phasespace(data, hor_axis, ver_axis, mesh, fix, slice)
    % plot phase space
    if nargin == 0 || isempty( data ); return; end;

    % is there data?
    if ~isfield(data.ps(data.my_axes),'phasespace') || isempty(data.ps(data.my_axes).phasespace)
        return
    end

    % unpack
    gammag  = data.ps(data.my_axes).gamma;
    x       = data.ps(data.my_axes).b_stat.x;
    b_emit  = data.ps(data.my_axes).b_stat.b_emit;
    b_twiss = data.ps(data.my_axes).b_stat.b_twiss;
    bin     = data.ps(data.my_axes).bin;
    grid    = data.ps(data.my_axes).b_stat.grid;
    valid   = [data.ps(data.my_axes).slices(:).valid];
    data.ps(data.my_axes).cbar_axes = [ ];
    
    % Is the slice toggle button pressed?
    if ( slice ~= -1 )&&( size(data.ps(data.my_axes).slices,2) >= slice )
        % replace phase space by slice
        i_slice = slice;
        if(data.ps(data.my_axes).slices(i_slice).valid)
            plot_ps = x((bin==i_slice),:);
            psv_message(data, ['Slice from ' num2str(grid(i_slice)) ' m to '...
                num2str(grid(i_slice+1)) ' m'],0);
            % use with slice parameters
            b_emit  =   data.ps(data.my_axes).slices(i_slice).b_stat.b_emit;
            b_twiss =   data.ps(data.my_axes).slices(i_slice).b_stat.b_twiss;
        else
            psv_message(data, ['Less than 10 particles in slice ' num2str(i_slice) '!'], 0);
            return
        end
    else
        plot_ps = x;
    end
 
    if(hor_axis <= 6)
        % if a phase space variable is plotted on the horizontal axis (and not the position along the beam line)
        if(ver_axis <= 6)
            %  for 'normal' phasespace display RMS values, emittances and
            %  Twiss parameters where fit
            % plot phase space
            btnDown = get(gca,'ButtonDownFcn');
            userData  = get(gca, 'UserData');
            cla;        
            if (mesh == 0)
                mi1 = min(min(plot_ps(:,hor_axis)));
                ma1 = max(max(plot_ps(:,hor_axis)));
                mi2 = min(min(plot_ps(:,ver_axis)));
                ma2 = max(max(plot_ps(:,ver_axis)));
                offset1 = abs( ma1 - mi1) * 0.05;
                offset2 = abs( ma2 - mi2) * 0.05;
                colordef white;
                colorbar('Off');
                line( plot_ps(:,hor_axis),plot_ps(:,ver_axis),'LineStyle', 'none', 'Marker', '.', 'ButtonDownFcn', btnDown, 'UserData', userData);
                if (fix == 1)||(ma1 == 0)||(ma2==0)
                    if (mi1~=0)&&(mi2~=0)
                        axis( [ min(mi1,mi2)-offset1 , max(ma1,ma2)+offset1 , min(mi1,mi2)-offset2 , max(ma1,ma2)+offset2 ] );
                    end
                else
                    axis( [ mi1-offset1 , ma1+offset1 , mi2-offset2 , ma2+offset2 ] );
                end
            else
                ps_mesh = [plot_ps(:,hor_axis) , plot_ps(:,ver_axis)];
                data.ps(data.my_axes).cbar_axes = psv_meshplot_phasespace(ps_mesh, data.psv_options(data.my_axes).plot.bin.nbins, fix);
            end;

            % for horizontal and vertical phase space, plot Twiss parameters and ellipse
            hold on;
            if (hor_axis == 1 && ver_axis == 2 || hor_axis == 3 && ver_axis == 4)
                alpha = b_twiss(1,hor_axis);
                beta =  b_twiss(2,hor_axis);
                gamma = b_twiss(3,hor_axis);
                % plot ellipse
                emit_real = b_emit(hor_axis)/gammag;
                %max_mesh = sqrt(emit_real * beta/(beta*gamma - alpha^2));
                max_mesh = sqrt(emit_real * beta);
                tpoints=1000;
                t_mesh = linspace(-max_mesh, max_mesh,tpoints);
                % shorten tmesh to avoid imaginary points
                t_mesh = t_mesh(2:(tpoints-1));
                line(t_mesh,-alpha/beta * t_mesh + sqrt(alpha^2/beta^2 * t_mesh.^2 -...
                    (gamma * t_mesh.^2 - emit_real)/beta),'LineStyle', 'none', 'Marker', '.','Color',[0 1 0], ...
                    'ButtonDownFcn', get(gca,'ButtonDownFcn'), 'UserData', userData);
                line(t_mesh,-alpha/beta * t_mesh - sqrt(alpha^2/beta^2 * t_mesh.^2 -...
                    (gamma * t_mesh.^2 - emit_real)/beta),'LineStyle', 'none', 'Marker', '.','Color',[0 1 0], ...
                    'ButtonDownFcn', get(gca,'ButtonDownFcn'), 'UserData', userData);
            end
            hold off;
            %
        elseif(ver_axis >= 7)
            % plot slice parameters vs. bunch position
            x_index = find(valid);
            plot_x = zeros(length(x_index),1);
            plot_y = zeros(length(x_index),1);
            switch ver_axis
                case 7      % plot current distribution
                    plot_x = data.ps(data.my_axes).Bcurrent(hor_axis).shape(:,1);
                    plot_y = data.ps(data.my_axes).Bcurrent(hor_axis).shape(:,2);
                case 8      % plot horizontal slice emittance
                    for i=x_index;
                        plot_x(i) = grid(i);
                        plot_y(i) = data.ps(data.my_axes).slices(i).b_stat.b_emit(1);
                    end
                case 9      % plot vertical slice emittance
                    for i=x_index;
                        plot_x(i) = grid(i);
                        plot_y(i) = data.ps(data.my_axes).slices(i).b_stat.b_emit(3);
                    end                
                case 10     % plot slice energy spread
                    for i=x_index;
                        plot_x(i) = grid(i);
                        plot_y(i) = data.ps(data.my_axes).slices(i).b_stat.pfit.e_rms;
                    end
                case 11     % plot longitudinal CSR field
                    if (isfield(data.ps(data.my_axes), 'fields')) && (~ isempty(data.ps(data.my_axes).fields))
                        leftEdge = grid(1) -1;
                        plot_y   = zeros(length(grid)-1,1);
                        plot_x   = plot_y;
                        for i=2:length(grid)
                            plot_y(i-1) = mean( data.ps(data.my_axes).fields ...
                                ( plot_ps(:,5) > leftEdge &(plot_ps(:,5) <= grid(i)) , 1 ) );
                            plot_x(i-1) = mean([ grid(i-1) grid(i) ]);
                            leftEdge = grid(i);
                        end
                    end
                case 12     % plot transverse CSR field
                    if (isfield(data.ps(data.my_axes), 'fields')) && (~ isempty(data.ps(data.my_axes).fields))
                        plot_x = grid;
                        plot_y = 1:length(grid);
                        %plot_x = grid;
                        %plot_y = data.ps(data.my_axes).fields(:,1) ; %E_trans; %TODO
                        
                        plotAndSetAxes(plot_x, plot_y);
                        plot_x = [ ];
                        hold on; 
                        for i=1:length(grid)-1
                            line( [grid(i) grid(i+1)], [i i],'Color',[1 0 0]);
                        end
                        hold off;
                        
                    end                
            end
            plotAndSetAxes(plot_x, plot_y);
        end
    elseif hor_axis == 7 % so called mesh plot
        plot_x = data.ps(data.my_axes).bl.mon_pos;
        if ver_axis == 1
            plot_y = data.ps(data.my_axes).mesh_data(:, 5);
        elseif ver_axis == 2
            plot_y = data.ps(data.my_axes).mesh_data(:, 6);
        elseif ver_axis == 3
            plot_y = data.ps(data.my_axes).mesh_data(:, 10);
        elseif ver_axis == 4
            plot_y = data.ps(data.my_axes).mesh_data(:, 7);
        elseif ver_axis == 5
            plot_y = data.ps(data.my_axes).mesh_data(:, 8);
        elseif ver_axis == 6
            plot_y = data.ps(data.my_axes).bl.current;
        end    
        plotAndSetAxes(plot_x, plot_y);
    end
end

function plotAndSetAxes(plotX, plotY)
        if isempty(plotX) || isempty(plotY)
            return
        end
        cla;
        colorbar('Off');
        line(plotX, plotY, 'ButtonDownFcn', get(gca,'ButtonDownFcn'), 'UserData', get(gca,'UserData'));
        minX = min(plotX);
        minY = min(plotY);
        maxX = max(plotX);
        maxY = max(plotY);
        if minX == maxX 
            minX = minX -1;
            maxX = maxX +1;
        end
        if minY == maxY
            minY = minY -1;
            maxY = maxY +1;
        end
        offsetX = abs( maxX - minX) * 0.05;
        offsetY = abs( maxY - minY) * 0.05;
        axis( [ minX-offsetX , maxX+offsetX, minY-offsetY , maxY+offsetY ] );
end