function [ data ] = psv_ps_statistics( data )
    if nargin ~= 1; return; end;
    grid=[ ];
    % is there data?
    if(~isfield(data.ps(data.my_axes),'phasespace'))
        return
    end

    % unpack ps data
    phasespace  = data.ps(data.my_axes).phasespace;
    gammag      = data.ps(data.my_axes).gamma;

    % subtract horizontal dispersion
    if data.psv_options(data.my_axes).plot.subtractDispersion == 1
        Dx  = polyfit(phasespace(:,6), phasespace(:,1), 2);
        Dxp = polyfit(phasespace(:,6), phasespace(:,2), 2);
        phasespace(:,1) = phasespace(:,1) - Dx(1) *phasespace(:,6).^2 -Dx (2)*phasespace(:,6) -Dx (3);
        phasespace(:,2) = phasespace(:,2) - Dxp(1)*phasespace(:,6).^2 -Dxp(2)*phasespace(:,6) -Dxp(3);
    end

    % get phase space parameters
    [x,b_mean,b_rms,b_emit,b_twiss,weight] = beam_statistics(phasespace,gammag);

    % put into structure 'ps'
    data.ps(data.my_axes).b_stat.x        = x;
    data.ps(data.my_axes).b_stat.b_mean   = b_mean;
    data.ps(data.my_axes).b_stat.b_rms    = b_rms;
    data.ps(data.my_axes).b_stat.b_emit   = b_emit;
    data.ps(data.my_axes).b_stat.b_twiss  = b_twiss;
    data.ps(data.my_axes).b_stat.weight   = weight;

    % after removing dispersion, this is the place for the binning...
    % get ps data
    Nsigma = 3;
    s = x(:,5);

    if data.psv_options(data.my_axes).plot.bin.equidistant
        grid_points = data.psv_options(data.my_axes).plot.bin.nbins;
        % grid for longitudinal slices
        grid = linspace(-Nsigma*b_rms(5),Nsigma*b_rms(5),grid_points)';
        
    elseif data.psv_options(data.my_axes).plot.bin.equalnum
        % alternative grid where each bin contains 'npart' particles
        nparts = data.psv_options(data.my_axes).plot.bin.nparts;
        if data.psv_options(data.my_axes).plot.bin.manstep
            stepsize =data.psv_options(data.my_axes).plot.bin.stepsize;
        else
            stepsize = b_rms(5)/100;
        end
        lolim   = min(s);
        hilim   = max(s);
        nsteps  = round((hilim-lolim)/stepsize); % step through in b_rms(5)/100 steps
        ind     = [ ];
        grid(1) = lolim;
        for i=1:nsteps
            s1=lolim + (i-1)/nsteps * (hilim-lolim);
            s2=lolim + i/nsteps * (hilim-lolim);
            h_ind = find(s>s1 & s<s2);
            ind = [ind h_ind'];
            if length(ind) > nparts
                grid = [grid s2];         
                ind =[ ];
            end
        end
        if grid(length(grid)) < hilim
            grid = [grid hilim];
        end
    
        if length(grid) <= 2
            psv_message(data, 'Only one bin contains all the particles! No slice analysis!',1);
            psv_message(data, 'Working...',0); 
        end
    else
        psv_message(data, 'Some sort of binning must be specified!',1);
        psv_message(data, 'Working...',0); 
        return
    end

    % save binning grid
    data.ps(data.my_axes).b_stat.grid = grid;
    % 'grid' contains the edges of the bins!
    data.ps(data.my_axes).nbins = length(grid)-1;

    % binning for the slice parameters
    [A,bin] = histc(s,grid);

    % store bin-id
    data.ps(data.my_axes).bin = bin;

    % calculate slice parameters
    data = psv_calc_slice(data) ;

    % calculate smoothed current curve
    if data.psv_options(data.my_axes).plot.bin.man_g_sigma
        sigma(1:6) = data.psv_options(data.my_axes).plot.bin.g_sigma;
    else
        sigma(1:6) = b_rms(:) ./ 10;
    end
    M = round( length(x(:,5)) / 100 ); 
    for i=1:6
        if sigma(i) ~= 0
            B = psv_smooth_I(x(:,i), M, sigma(i), data.ps(data.my_axes).q_total, 3e8);
        else
            B = zeros(100,2);
        end
        data.ps(data.my_axes).Bcurrent(i).shape = B;
    end
end

