function data = psv_set_ui_visibility( data, updateSlice )
% set the ui-elements properties depending on phasespace
    if nargin == 0; return; end;    
    if nargin > 1
        if updateSlice
            data = updateSliceOnly( data, 1);
            data = updateSliceOnly( data, 2);
        end
        return;
    end
    
    for axesIndex=1:2
        meshMode   = 'Off';
        squareMode = 'Off';
        fileSlide  = 'Off';
        sliceMode  = 'Off';
        sliceToggle = 'Off';
        popupMode = 'Off';
        rmsMode   = 'Off';
        emitMode  = 'Off';
        twissMode = 'Off';
        rms1Value  = ' ';
        rms2Value  = ' ';
        twissValue = ' ';
        emitValue  = ' ';
        slicePos = 0;
        if (isPhaseSpace(data, axesIndex))       % is phasespace loaded
            meshMode = 'On';
            squareMode = 'On';
            popupMode = 'On';
            if ( isFileNames(data, axesIndex))  % are there more files detected
                fileSlide = 'On';
                [ fileMax fileStep filePos ] = getFileSlideParameter(data, axesIndex);
                set(data.psv_hnames.dir_handles(1,axesIndex),'Min', 1);
                set(data.psv_hnames.dir_handles(1,axesIndex),'Max', fileMax);
                set(data.psv_hnames.dir_handles(1,axesIndex),'Sliderstep', fileStep);
                set(data.psv_hnames.dir_handles(1,axesIndex),'Value', filePos);
                set(data.psv_hnames.dir_handles(2,axesIndex),'String',num2str(filePos));
            end
            isSlice   = false;
            if (data.psv_hnames.isSliceable(data, axesIndex))   % is phasespace sliceable
                sliceToggle = 'On';
                if(get(data.psv_hnames.slice_handles(6,axesIndex),'Value'))
                    % enable slice selector if toggle button is pressed
                    sliceMode = 'On';
                    isSlice   = true;
                    [sliceMin sliceMax sliceStep slicePos] ...
                        = getSliceParameter(data, axesIndex);
                    set(data.psv_hnames.slice_handles(1,axesIndex),'Min', sliceMin);
                    set(data.psv_hnames.slice_handles(1,axesIndex),'Max', sliceMax);
                    set(data.psv_hnames.slice_handles(1,axesIndex),'Sliderstep', sliceStep);
                    set(data.psv_hnames.slice_handles(1,axesIndex),'Value',slicePos);
                    set(data.psv_hnames.slice_handles(2,axesIndex),'String',num2str(slicePos));
                    set(data.psv_hnames.slice_handles(3,axesIndex),'String',num2str(sliceMax));
                end
            end
            % get rms/emit/twiss values
            [hor_axis ver_axis] = getAxis(data, axesIndex);
            [rmsMode emitMode twissMode rms1Value rms2Value twissValue emitValue] ...
                = get_rms(data, axesIndex, hor_axis, ver_axis, sliceMode,slicePos);
            % setup popup menu entries
            [popupString1 popupString2] = data.psv_hnames.getPopupEntries(data, axesIndex, isSlice);
            set(data.psv_hnames.popup_handles(axesIndex,1),'String',popupString1);
            set(data.psv_hnames.popup_handles(axesIndex,2),'String',popupString2);
            set(data.psv_hnames.popup_handles(axesIndex,1), 'Value', ver_axis);
            set(data.psv_hnames.popup_handles(axesIndex,2), 'Value', hor_axis);
            % disable meshplot/fix axes/toggle slice if we are in any line plot mode
            if ( hor_axis > 6 ) || ( ver_axis > 6 )
                meshMode = 'Off'; squareMode = 'Off'; sliceToggle = 'Off'; sliceMode = 'Off';
            end
            if hor_axis == 7 % disable file slider if we are in mesh plot mode
                fileSlide = 'Off'; 
            end
        end
        set(data.psv_hnames.mesh_handles(:,axesIndex),'Visible',meshMode);
        set(data.psv_hnames.square_handles(:,axesIndex),'Visible',squareMode);
        set(data.psv_hnames.dir_handles(:,axesIndex),'Visible',fileSlide);
        set(data.psv_hnames.slice_handles(1:5,axesIndex),'Visible',sliceMode);
        set(data.psv_hnames.slice_handles(6,axesIndex),'Visible',sliceToggle);
        set(data.psv_hnames.popup_handles(axesIndex,:),'Visible',popupMode);
        set(data.psv_hnames.RMS_handles(axesIndex),'Visible', rmsMode)
        set(data.psv_hnames.emit_handles(axesIndex), 'Visible', emitMode)
        set(data.psv_hnames.twiss_handles(axesIndex),'Visible', twissMode)
        set(data.psv_hnames.RMS_handles(axesIndex,2),'String', rms1Value );
        set(data.psv_hnames.RMS_handles(axesIndex,1),'String', rms2Value );
        set(data.psv_hnames.emit_handles(axesIndex),'String', emitValue);
        set(data.psv_hnames.twiss_handles(axesIndex),'String', twissValue);
        set(data.psv_hnames.axes_handles(axesIndex),'Color', getAxisColor(data, axesIndex));
    end
    menuStatus = 'on';
    if ~ hasCurrentData( data )
        menuStatus = 'off';
    end
    menuMeshPlotStatus = 'on';
    if ~ isMeshPlotPossible(data)
        menuMeshPlotStatus = 'off';
    end
    set(data.psv_hnames.menus(1,:), 'Enable', menuStatus );
    set(data.psv_hnames.meshplotMenu, 'Enable', menuMeshPlotStatus );
    set(data.psv_hnames.poltminchi, 'Enable', menuStatus );
    set(data.psv_hnames.psv_gui,'Name', getWindowName(data));
end

function data = updateSliceOnly(data, axesIndex)
    sliceMode = '';
    slicePos = 0;
    if ~isPhaseSpace(data, axesIndex)
        return;
    end
        
    if (data.psv_hnames.isSliceable(data, axesIndex))   % is phasespace sliceable
        if(get(data.psv_hnames.slice_handles(6,axesIndex),'Value'))
            % enable slice selector if toggle button is pressed
            sliceMode = 'On';
            slicePos = get(data.psv_hnames.slice_handles(1,axesIndex),'Value');
        end
    end
    % get rms/emit/twiss values
    [hor_axis ver_axis] = getAxis(data, axesIndex);
    [rmsMode emitMode twissMode rms1Value rms2Value twissValue emitValue] ...
        = get_rms(data, axesIndex, hor_axis, ver_axis, sliceMode,slicePos);
    
    set(data.psv_hnames.emit_handles(axesIndex),'String', emitValue);
    set(data.psv_hnames.twiss_handles(axesIndex),'String', twissValue);
end

function result=isPhaseSpace(data, index)
    result = false;
    if (length(data.ps) >= index)    
        if (isfield(data.ps(index), 'phasespace'))
            if (~ isempty(data.ps(index).phasespace))
                result = true;
             end
        end
    end
end
     
function result=isFileNames(data, index)
    result = false;
    if (isfield(data.ps(index), 'bl'))
        if (isfield(data.ps(index).bl, 'file_names'))
            if (~ isempty(data.ps(index).bl.file_names))
                result = true;
            end
        end
    end
end
 
function [rmsMode emitMode twissMode rms1Value rms2Value twissValue emitValue] ...
            = get_rms(data, index, hor_axis, ver_axis, sliceMode , slicePos)
    rmsMode    = 'Off';
    emitMode   = 'Off';
    twissMode  = 'Off';
    rms1Value  = ' ';
    rms2Value  = ' ';
    twissValue = ' ';
    emitValue  = ' ';
    if (hor_axis <= 6)
        % if a phase space variable is plotted on the horizontal axis (and not the position along the beam line)
        if (ver_axis <= 6) && (isfield(data.ps(index),'b_stat'))
            %  for 'normal' phasespace display RMS values, emittances and
            %  Twiss parameters where fit
            b_rms   = data.ps(index).b_stat.b_rms;
            b_emit  = data.ps(index).b_stat.b_emit;
            b_twiss = data.ps(index).b_stat.b_twiss;
            unit    = data.ps(index).unit;
            unit_e  = data.ps(index).unit_e;
            rmsMode = 'On';
            % display RMS values, emittance and optical functions
            rms1Value  = ['RMS: ' num2str(b_rms(hor_axis) ,'%0.2e'),' ',char(unit(hor_axis))] ;
            rms2Value  = ['RMS: ' num2str(b_rms(ver_axis) ,'%0.2e'),' ',char(unit(ver_axis))] ;
            % display emittance for the usual projections
            if (hor_axis == 1 && ver_axis == 2 || hor_axis == 3 && ver_axis == 4 || hor_axis == 5 && ver_axis == 6)
                emitMode = 'On';
                if strcmp( sliceMode, 'On' )
                    em = data.ps(index).slices(slicePos).b_stat.b_emit(1,hor_axis);
                else
                    em = b_emit(hor_axis);
                end
                emitValue = ['Emittance: ' num2str(em,'%0.2e') ' ' char(unit_e(hor_axis))];
            end
            % for horizontal and vertical phase space, plot Twiss parameters and ellipse
            if (hor_axis == 1 && ver_axis == 2 || hor_axis == 3 && ver_axis == 4)
                twissMode = 'On';
                if strcmp( sliceMode, 'On' )
                    alpha = data.ps(index).slices(slicePos).b_stat.b_twiss(1,hor_axis);
                    beta =  data.ps(index).slices(slicePos).b_stat.b_twiss(2,hor_axis);
                else
                    alpha = b_twiss(1,hor_axis);
                    beta =  b_twiss(2,hor_axis);
                end
                twissValue = ['beta = ',num2str(beta,'%0.3g'), ' alpha = ',num2str(alpha,'%0.3g')];
            end
        end
    end
end

function [sliceMin sliceMax sliceStep slicePos] = getSliceParameter(data, index)
    % read number of slices
    nr_of_slices = data.ps(index).nbins;
    sliceMin   = 1;
    sliceMax   = nr_of_slices;
    sliceStep  = [1/(nr_of_slices-1) 5/(nr_of_slices-1)];
    slicePos   = get(data.psv_hnames.slice_handles(1,index),'Value');
    if slicePos > sliceMax
        slicePos = sliceMax;
    end
end

function [hor_axis ver_axis] = getAxis(data, index)
    hor_axis = data.ps(index).hor_axis;
    ver_axis = data.ps(index).ver_axis;
end

function [ fileMax fileStep filePos ] = getFileSlideParameter(data, index)
    ps_files = data.ps(index).bl.file_names;
    fileMax  = length(ps_files);
    fileStep = [1/(length(ps_files)-1) 10/(length(ps_files)-1)];
    filePos  = data.ps(index).bl.act_file;
end

function windowName = getWindowName( data )
    windowName = [ 'CSRtrack Phase Space Viewer (' num2str(data.my_axes) ')'];
    if ~ isPhaseSpace(data, data.my_axes)
        return;
    end
    if ~isfield(data.ps(data.my_axes),'filename')
        return;
    end
    if isempty(data.ps(data.my_axes).filename)
        return;
    end
    windowName = [ windowName ' ' data.ps(data.my_axes).filename];
end

function axesColor = getAxisColor(data, index)
    if index == data.my_axes
        axesColor = [ 1., 1., 1.];
    else
        axesColor = [0.9,0.9,0.9];
    end
end

function result = enabledMingChi( data )
    result = true;
    if ~ hasCurrentData(data)
        result = false;
        return;
    end
    if data.psv_options(data.my_axes).plot.MeshPlot.MingChi.yn ~= 1
        result = false;
    end
end

function result = hasCurrentData( data )
    result = true;
    if ~ isPhaseSpace(data, data.my_axes)
        result = false;
    end
end

function result = isMeshPlotPossible( data )
    result = false;
    if ~ hasCurrentData(data)
        return
    end
    if ~ isfield(data.ps(data.my_axes),'bl')
        return
    end
    if isempty(data.ps(data.my_axes).bl)
        return
    end
    result = true;
end
