function data = psv_sum_plot( data )
    if nargin == 0; return ; end;
%
% if structure ps does carry beam data return 
if(isfield(data.ps(data.my_axes),'b_stat'))
else
    uiwait(msgbox('No phase space data!','PSV_SUM_PLOT','error'))
    return
end
%
% unpack
phasespace  = data.ps(data.my_axes).phasespace;
x           = data.ps(data.my_axes).b_stat.x;
gammag      = data.ps(data.my_axes).gamma;
ct          = data.ps(data.my_axes).ct;
b_mean      = data.ps(data.my_axes).b_stat.b_mean;
b_rms       = data.ps(data.my_axes).b_stat.b_rms;
b_emit      = data.ps(data.my_axes).b_stat.b_emit;
%
for i=1:6
 offset(i) = mean(x(:,i)-phasespace(:,i));
end
%
%--------------------------------------------------------------------------
% make plot
figure
set(gcf,'Position',[60 60 680 890])
%
grid = data.ps(data.my_axes).b_stat.grid;
valid = [data.ps(data.my_axes).slices(:).valid];
x_index = find(valid);
%
subplot(4,2,1)
% plot current distribution -----------------------------------
Bshape = data.ps(data.my_axes).Bcurrent(5).shape;
plot(Bshape(:,1), Bshape(:,2));
htit1 = title('Longitudinal Current Profile');
hyl1 = ylabel('Bunch Current [A]');
hxl1 = xlabel('s (along the Bunch) [m]');
%
% find maximum and half-maximum positions
%
[peak_current i_max] = max(Bshape(:,2));
b_center = Bshape(i_max,1);
% find grid index of center
[dummy center_grid_index] = min(abs(b_center-grid));
if(b_center - grid(center_grid_index) <= 0);
    center_grid_index = center_grid_index -1;
end
% find half-max before
[dummy i_half_1] = min(abs(peak_current/2 - ...
    Bshape(1:i_max-1,2)));
% after
[dummy i_half_2] = min(abs(peak_current/2 - ...
    Bshape( i_max+1 : length(Bshape(:,2)),2) ));
i_half_2 = i_half_2 + i_max;
%
% find border positions of half-max region
i_half_1_x = Bshape(i_half_1,1);
i_half_2_x = Bshape(i_half_2,1);
%
% RMS values for above half max region
ind_hmax = find(x(:,5)>i_half_1_x &  x(:,5)<i_half_2_x);
%
b_rms_hmax = std(x(ind_hmax,:));
% draw vertical lines
hold on
y_lim = get(gca,'YLim');
plot([i_half_1_x i_half_1_x], [0 y_lim(2)],'g--');
plot([i_half_2_x i_half_2_x], [0 y_lim(2)],'g--');
%
% axis values for horizontal axis
if(data.psv_options(data.my_axes).plot.SumPlot.center_peak)
    s_width = max(abs(b_center - i_half_1_x),abs(b_center - i_half_2_x))...
        + (data.psv_options(data.my_axes).plot.SumPlot.s_window_width-1)*abs(i_half_1_x - i_half_2_x)/2;
    %
    grid_axis = [(b_center - s_width)  (b_center + s_width)];
    s_axis = axis;
    axis([grid_axis s_axis(3:4)]);
else
    grid_axis = axis;
    grid_axis = grid_axis(1:2);
end
%
%
subplot(4,2,3)
% horizontal and vertical slice emittance----------------------------------
for i=x_index;
    plot_x (i) = (grid(i) + grid(i+1))/2;
    plot_y1(i) = data.ps(data.my_axes).slices(i).b_stat.b_emit(1);
    plot_y2(i) = data.ps(data.my_axes).slices(i).b_stat.b_emit(3);
end
plot(plot_x,plot_y1,'b-');
hold on
plot(plot_x,plot_y2,'m-');
%
x_lim = grid_axis;
y_lim = [0 data.psv_options(data.my_axes).plot.SumPlot.sl_e_lim];
set(gca,'XLim',x_lim);
set(gca,'YLim',y_lim);
y_lim = get(gca,'YLim');
plot([i_half_1_x i_half_1_x], [0 y_lim(2)],'g--');
plot([i_half_2_x i_half_2_x], [0 y_lim(2)],'g--');
%
htit2 = title('Slice Emittance');
hyl2 = ylabel('Emittance [m]');
hxl2 = xlabel('s [m]');
%
% make legend, because matlab legend cannot be reformatted (matlab bug)
htext1 = text(0.84*x_lim(2),0.94*y_lim(2),'hor.');
htext2 = text(0.84*x_lim(2),0.87*y_lim(2),'ver.');
set([htext1 htext2],'Fontsize',8);
set(htext1,'Color',[0 0 1]);
set(htext2,'Color',[1 0 1]);    
%    
%
subplot(4,2,5)
% slice energy spread -----------------------------------------------------
for i=x_index;
    plot_x(i) = (grid(i) + grid(i+1))/2;
    plot_y(i) = data.ps(data.my_axes).slices(i).b_stat.pfit.e_rms *gammag*0.511;
end
plot(plot_x,plot_y,'b-');
htit4 = title('Slice Energy Spread');
hyl4  = ylabel('Energy Spread [MeV]');
hxl4  = xlabel('s [m]');
set(gca,'XLim',grid_axis);
%
hold on
y_lim = get(gca,'YLim');
plot([i_half_1_x i_half_1_x], [0 y_lim(2)],'g--');
plot([i_half_2_x i_half_2_x], [0 y_lim(2)],'g--');
%
%
%
subplot(4,2,2)
% longitudinal phase space ------------------------------------------------
h = plot(x(:,5),x(:,6),'bo');
hold on
% plot centroids of slices 
for i=x_index;
    plot_x(i) = (grid(i) + grid(i+1))/2;
    plot_y(i) = data.ps(data.my_axes).slices(i).b_stat.b_mean(6);
end
plot(plot_x,plot_y,'r-');
%
htit3 = title('Longitudinal Phase Space');
hyl3  = ylabel('de [relative]');
hxl3  = xlabel('s [m]');
set(gca,'XLim',grid_axis);
set(h,'Markersize',1)
%
%
subplot(4,2,7)
% beta beat ---------------------------------------------------------------
%
% twiss parameters of bunch slices
for i=x_index
    sl_twiss(i,:,:) = data.ps(data.my_axes).slices(i).b_stat.b_twiss;
end
for i=x_index
    Mx(i) = 0.5 * (sl_twiss(center_grid_index,2,1)*sl_twiss(i,3,1)...
        - 2* sl_twiss(center_grid_index,1,1)*sl_twiss(i,1,1)...
        + sl_twiss(i,2,1)*sl_twiss(center_grid_index,3,1));
    My(i) = 0.5 * (sl_twiss(center_grid_index,2,3)*sl_twiss(i,3,3)...
        - 2* sl_twiss(center_grid_index,1,3)*sl_twiss(i,1,3)...
        + sl_twiss(i,2,3)*sl_twiss(center_grid_index,3,3));
end
for i=x_index
    plot_x(i)  = (grid(i) + grid(i+1))/2;
    % for M=1, M**2-1 can be slightly negative due to rounding errors, so
    % take absolute value
    plot_y1(i) = Mx(i) + sqrt(abs(Mx(i).^2 -1));
    plot_y2(i) = My(i) + sqrt(abs(My(i).^2 -1));
end
%
plot(plot_x,plot_y1,'b-');
hold on
plot(plot_x,plot_y2,'m-');  
%
x_lim = grid_axis;
y_lim = [0 data.psv_options(data.my_axes).plot.SumPlot.betab_lim];
set(gca,'XLim',x_lim);
set(gca,'YLim',y_lim);
y_lim = get(gca,'YLim');
plot([i_half_1_x i_half_1_x], [0 y_lim(2)],'g--');
plot([i_half_2_x i_half_2_x], [0 y_lim(2)],'g--');
%
htit5 = title('Beta Mismatch relative to Bunch Center');
hyl5  = ylabel('Beta Mismatch');
hxl5  = xlabel('s [m]');
%
% make legend, because matlab legend cannot be reformatted (matlab bug)
htext1 = text(0.84*x_lim(2),0.94*y_lim(2),'hor.');
htext2 = text(0.84*x_lim(2),0.87*y_lim(2),'ver.');
set([htext1 htext2],'Fontsize',8);
set(htext1,'Color',[0 0 1]);
set(htext2,'Color',[1 0 1]);  
%
%
%
subplot(4,2,8)
% text --------------------------------------------------------------------
set(gca,'Visible','off');
%
vtpos = 0.99;
t_incr = 0.13;
%text(0.,vtpos,['Summary Plot on ' date ],'Fontsize',8);
% % split path in pieces of less than 65 characters 
% file_line = char(data.ps(data.my_axes).filename);
% slash_pos = strfind(file_line,'\');
% break_pos = slash_pos(max(find(slash_pos<65)));
% %
% line_1    = file_line(1:break_pos);
% text(0.,vtpos,line_1,'Interpreter','none','Fontsize',8);
% vtpos = vtpos - t_incr;
% if(length(file_line) > break_pos)
%     line_2    = file_line(break_pos+1:length(file_line));
%     text(0.,vtpos,line_2,'Interpreter','none','Fontsize',8);
%     vtpos = vtpos - t_incr;
% end
%
text(0.,vtpos,['Number of Particles: ' num2str(length(phasespace))...
    '   Charge: ' num2str(data.ps(data.my_axes).q_total*1e9,'%3.1g') ' nC'],'Fontsize',8);
vtpos = vtpos - t_incr;
text(0.,vtpos,['Position: ' num2str(ct,5) ' m    Beam Energy: ' num2str(gammag*0.511,5) ' MeV'],'Fontsize',8);
vtpos = vtpos - 1.5*t_incr;
%
FWHM = abs(i_half_2_x-i_half_1_x)*1e6; % full width half max in micro-meters
text(0.,vtpos,['FWHM (distance between green bars): ' num2str(FWHM,3) ' \mu m '...
    '(' num2str(FWHM/3e8*1e6,3) ' ps)'],'Fontsize',7);
vtpos = vtpos - t_incr;
%
charge_FWHM = length(find(x(:,5) > i_half_1_x  &  x(:,5) <i_half_2_x))...
    /length(x(:,5)) * 100; % charge in full width half max in per cent
text(0.,vtpos,['Charge within FWHM: ' num2str(charge_FWHM,3) ' %'],'Fontsize',7);
vtpos = vtpos - t_incr;
%
text(0.,vtpos,['Projected Emittance: \gamma\epsilon_x = ' num2str(b_emit(1),3) ' m'...
    ' \gamma\epsilon_y = ' num2str(b_emit(3),3) ' m'...
    ],'Fontsize',7);
vtpos = vtpos - t_incr;
%
% optic values at current peak
mydigits = 3;
text(0.,vtpos,['Optics @ I_{peak}: \alpha_x = '...
                num2str(sl_twiss(center_grid_index,1,1),mydigits)...
 ' \beta_x = '   num2str(sl_twiss(center_grid_index,2,1),mydigits) 'm'...
 ' \alpha_y = '  num2str(sl_twiss(center_grid_index,1,3),mydigits)...
 ' \beta_y = '   num2str(sl_twiss(center_grid_index,2,3),mydigits) 'm'...
 ],...
 'Fontsize',7);
vtpos = vtpos - 1.5*t_incr;
%
% RMS values
text(0.,vtpos,'RMS Values for all Particles:','Fontsize',7);
%
vtpos = vtpos - t_incr;
mydigits0 = '%4.2d';
fontsize0=8;
text(0.,vtpos,['     '...
    ' x = ' num2str(b_rms(1),mydigits0) ' m  x'' = '  num2str(b_rms(2),mydigits0)...
    ],'FontName','Courier New','Fontsize',fontsize0);
vtpos = vtpos - 0.7*t_incr;
text(0.,vtpos,['     '...
' y = ' num2str(b_rms(3),mydigits0) ' m  y'' = '  num2str(b_rms(4),mydigits0)...
     ],'FontName','Courier New','Fontsize',fontsize0);
vtpos = vtpos - 0.7*t_incr;
text(0.,vtpos,['     '...
 ' s = ' num2str(b_rms(5),mydigits0) ' m  \delta  = '  num2str(b_rms(6),mydigits0)...
     ],'FontName','Courier New','Fontsize',fontsize0);
vtpos = vtpos - t_incr;
text(0.,vtpos,'RMS Values within FWHM:','Fontsize',7);
vtpos = vtpos - t_incr;
%
text(0.,vtpos,['     '...
    ' x = ' num2str(b_rms_hmax(1),mydigits0) ' m  x'' = '  num2str(b_rms_hmax(2),mydigits0)...
    ],'FontName','Courier New','Fontsize',fontsize0);
vtpos = vtpos - 0.7*t_incr;
text(0.,vtpos,['     '...
 ' y = ' num2str(b_rms_hmax(3),mydigits0) ' m  y'' = '  num2str(b_rms_hmax(4),mydigits0)...
     ],'FontName','Courier New','Fontsize',fontsize0);
vtpos = vtpos - 0.7*t_incr;
text(0.,vtpos,['     '...
 ' s = ' num2str(b_rms_hmax(5),mydigits0) ' m  \delta  = '  num2str(b_rms_hmax(6),mydigits0)...
     ],'FontName','Courier New','Fontsize',fontsize0);
%
%
subplot(4,2,4)
% top view ----------------------------------------------------------------
h = plot(x(:,5),x(:,1),'bo');
hold on
for i=x_index;
    plot_x(i) = (grid(i) + grid(i+1))/2;
    plot_y(i) = data.ps(data.my_axes).slices(i).b_stat.b_mean(1);
end
plot(plot_x,plot_y,'r-');
%
htit7 = title('Top View');
hyl7  = ylabel('Horizontal Position [m]');
hxl7  = xlabel('s [m]');
set(h,'Markersize',1);
set(gca,'XLim',grid_axis);
%
%
subplot(4,2,6)
% side view ---------------------------------------------------------------
h = plot(x(:,5),x(:,3),'bo');
hold on
for i=x_index;
    plot_x(i) = (grid(i) + grid(i+1))/2;
    plot_y(i) = data.ps(data.my_axes).slices(i).b_stat.b_mean(3);
end
plot(plot_x,plot_y,'r-');
%
htit8 = title('Side View');
hyl8  = ylabel('Vertical Position [m]');
hxl8  = xlabel('s [m]');
set(h,'Markersize',1);
set(gca,'XLim',grid_axis);
%
% adjust fonts etc --------------------------------------------------------
set([hxl1 hxl2 hxl3 hxl4 hxl5 hxl7 hxl8 hyl1 hyl2 hyl3 hyl4 hyl5 hyl7 hyl8],'Fontsize',8);
set([htit1 htit2 htit3 htit4 htit5 htit7 htit8 ],'Fontname','Times');
